@@ -22,6 +22,9 @@ class cache {
 		std::vector<rss_item> search_for_items(const std::string& querystr, const std::string& feedurl);
 		rss_feed get_feed_by_url(const std::string& feedurl);
 		void catchup_all(const std::string& feedurl = "");
+		void catchup_all(rss_feed& feed);
+		void update_rssitem_flags(rss_item& item);
+		std::vector<std::string> get_feed_urls();
 	private:
 		void populate_tables();
 		void set_pragmas();
