@@ -2,6 +2,8 @@
 #define NEWSBEUTER_FEEDLIST_FORMACTION__H
 
 #include <formaction.h>
+#include <matcher.h>
+#include <history.h>
 
 namespace newsbeuter {
 
@@ -9,7 +11,6 @@ class feedlist_formaction : public formaction {
 	public:
 		feedlist_formaction(view *, std::string formstr);
 		virtual ~feedlist_formaction();
-		virtual void process_operation(operation op);
 		virtual void prepare();
 		virtual void init();
 		void set_feedlist(std::vector<rss_feed>& feeds);
@@ -17,10 +18,18 @@ class feedlist_formaction : public formaction {
 		virtual keymap_hint_entry * get_keymap_hint();
 		rss_feed * get_feed();
 
-		bool jump_to_next_unread_feed();
+		bool jump_to_next_unread_feed(unsigned int& feedpos);
+		bool jump_to_previous_unread_feed(unsigned int& feedpos);
+
+		virtual void handle_cmdline(const std::string& cmd);
+
+		virtual void finished_qna(operation op);
 
 	private:
 
+		int get_pos(unsigned int realidx);
+		virtual void process_operation(operation op);
+
 		bool zero_feedpos;
 		unsigned int feeds_shown;
 		bool auto_open;
@@ -28,6 +37,13 @@ class feedlist_formaction : public formaction {
 		std::vector<std::pair<rss_feed *, unsigned int> > visible_feeds;
 		std::string tag;
 		std::vector<std::string> tags;
+
+		matcher m;
+		bool apply_filter;
+
+		history filterhistory;
+
+		rss_feed search_dummy_feed;
 };
 
 }
