@@ -11,7 +11,6 @@ class filebrowser_formaction : public formaction {
 	public:
 		filebrowser_formaction(view *, std::string formstr);
 		virtual ~filebrowser_formaction();
-		virtual void process_operation(operation op);
 		virtual void prepare();
 		virtual void init();
 		virtual keymap_hint_entry * get_keymap_hint();
@@ -21,9 +20,8 @@ class filebrowser_formaction : public formaction {
 		inline void set_type(filebrowser_type t) { type = t; }
 
 	private:
+		virtual void process_operation(operation op);
 
-		std::string fancy_quote(const std::string& s);
-		std::string fancy_unquote(const std::string& s);
 		std::string add_file(std::string filename);
 		std::string get_filename_suggestion(const std::string& s);
 		std::string get_rwx(unsigned short val);
