@@ -12,6 +12,8 @@
 
 #define KM_NEWSBEUTER (1<<0)
 #define KM_PODBEUTER  (1<<1)
+#define KM_SYSKEYS    (1<<2)
+#define KM_INTERNAL   (1<<3)
 #define KM_BOTH (KM_NEWSBEUTER | KM_PODBEUTER)
 
 namespace newsbeuter {
@@ -27,6 +29,7 @@ namespace newsbeuter {
 						OP_OPEN, 
 						OP_SAVE, 
 						OP_NEXTUNREAD, 
+						OP_PREVUNREAD,
 						OP_OPENINBROWSER, 
 						OP_HELP, 
 						OP_TOGGLESOURCEVIEW, 
@@ -38,6 +41,15 @@ namespace newsbeuter {
 						OP_SEARCH, 
 						OP_ENQUEUE,
 						OP_REDRAW,
+						OP_CMDLINE,
+						OP_SETFILTER,
+						OP_CLEARFILTER,
+						OP_SELECTFILTER,
+						OP_RELOADURLS,
+						OP_BOOKMARK,
+						OP_EDITFLAGS,
+						OP_NEXTFEED,
+						OP_PREVFEED,
 						OP_NB_MAX,
 
 						// podbeuter-specific operations:
@@ -50,11 +62,41 @@ namespace newsbeuter {
 						OP_PB_MOREDL, 
 						OP_PB_LESSDL, 
 						OP_PB_PLAY, 
-						OP_PB_MAX };
+						OP_PB_MAX,
+						
+						OP_SK_MIN,
+						OP_SK_UP,
+						OP_SK_DOWN,
+						OP_SK_PGUP,
+						OP_SK_PGDOWN,
+						/* TODO: add more user-defined keys here */
+						OP_SK_MAX,
+
+						OP_INT_MIN,
+
+						OP_INT_END_CMDLINE,
+						OP_INT_END_SETFILTER,
+						OP_INT_BM_END,
+						OP_INT_EDITFLAGS_END,
+						OP_INT_START_SEARCH,
+
+						OP_INT_END_QUESTION,
+						OP_INT_CANCEL_QNA,
+						OP_INT_QNA_NEXTHIST,
+						OP_INT_QNA_PREVHIST,
+
+						OP_INT_MAX
+						};
+
+	struct keymap_desc {
+		std::string key;
+		std::string cmd;
+		std::string desc;
+	};
 
 	class keymap : public config_action_handler {
 		public:
-			keymap();
+			keymap(unsigned int flags);
 			~keymap();
 			void set_key(operation op, const std::string& key);
 			void unset_key(const std::string& key);
@@ -63,7 +105,7 @@ namespace newsbeuter {
 			char get_key(const std::string& keycode);
 			std::string getkey(operation );
 			virtual action_handler_status handle_action(const std::string& action, const std::vector<std::string>& params);
-			void get_keymap_descriptions(std::vector<std::pair<std::string,std::string> >& descs, unsigned short flags);
+			void get_keymap_descriptions(std::vector<keymap_desc>& descs, unsigned short flags);
 		private:
 			std::map<std::string,operation> keymap_;
 	};
