@@ -20,7 +20,11 @@ class pb_view {
 		pb_view(pb_controller * c = 0);
 		~pb_view();
 		void run();
-		void set_keymap(newsbeuter::keymap * k) { keys = k; }
+		void set_keymap(newsbeuter::keymap * k) { 
+			keys = k; 
+			set_bindings();
+		}
+
 	private:
 
 		friend class newsbeuter::colormanager;
@@ -33,6 +37,8 @@ class pb_view {
 		void run_help();
 		void set_dllist_keymap_hint();
 		void set_help_keymap_hint();
+		void set_bindings();
+
 		std::string prepare_keymaphint(keymap_hint_entry * hints);
 		pb_controller * ctrl;
 		newsbeuter::stfl::form dllist_form;
