@@ -22,8 +22,13 @@ class utils {
 
 		static std::string get_command_output(const std::string& cmd);
 		static void extract_filter(const std::string& line, std::string& filter, std::string& url);
-		static std::string retrieve_url(const std::string& url, const char * user_agent = 0);
+		static std::string retrieve_url(const std::string& url, const char * user_agent = 0, const char * auth = 0);
 		static std::string run_filter(const std::string& cmd, const std::string& input);
+		static void run_command(const std::string& cmd, const std::string& param); // used for notifications only
+		static std::string run_program(char * argv[], const std::string& input); // used for bookmarking
+
+		static std::string resolve_tilde(const std::string& );
+		static std::string replace_all(std::string str, const std::string& from, const std::string& to);
 };
 
 }
