@@ -10,6 +10,7 @@
 #include <rss.h>
 #include <logger.h>
 #include <config.h>
+#include <exceptions.h>
 
 using namespace newsbeuter;
 
@@ -49,12 +50,22 @@ static int rssfeed_callback(void * myfeed, int argc, char ** argv, char ** /* az
 	return 0;
 }
 
+static int vectorofstring_callback(void * vp, int argc, char ** argv, char ** /* azColName */) {
+	std::vector<std::string> * vectorptr = (std::vector<std::string> *)vp;
+	assert(argc == 1);
+	assert(argv[0] != NULL);
+	vectorptr->push_back(std::string(argv[0]));
+	GetLogger().log(LOG_INFO, "vectorofstring_callback: element = %s", argv[0]);
+	return 0;
+}
+
 static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
 	rss_feed * feed = (rss_feed *)myfeed;
-	assert (argc == 11);
+	assert (argc == 12);
 	rss_item item(NULL);
 	item.set_guid(argv[0]);
 	item.set_title(argv[1]);
+	GetLogger().log(LOG_DEBUG, "rssitem_callback: title = %s", argv[1]);
 	item.set_author(argv[2]);
 	item.set_link(argv[3]);
 	
@@ -71,6 +82,7 @@ static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* az
 	item.set_enclosure_url(argv[8] ? argv[8] : "");
 	item.set_enclosure_type(argv[9] ? argv[9] : "");
 	item.set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
+	item.set_flags(argv[11] ? argv[11] : "");
 
 	feed->items().push_back(item);
 	return 0;
@@ -79,7 +91,7 @@ static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* az
 static int rssitemvector_callback(void * vector, int argc, char ** argv, char ** /* azColName */) {
 	std::vector<rss_item> * items = (std::vector<rss_item> *)vector;
 
-	assert (argc == 11);
+	assert (argc == 12);
 	rss_item item(NULL);
 	item.set_guid(argv[0]);
 	item.set_title(argv[1]);
@@ -99,6 +111,7 @@ static int rssitemvector_callback(void * vector, int argc, char ** argv, char **
 	item.set_enclosure_url(argv[8] ? argv[8] : "");
 	item.set_enclosure_type(argv[9] ? argv[9] : "");
 	item.set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
+	item.set_flags(argv[11] ? argv[11] : "");
 
 	items->push_back(item);
 	return 0;
@@ -106,7 +119,7 @@ static int rssitemvector_callback(void * vector, int argc, char ** argv, char **
 
 static int search_item_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
 	std::vector<rss_item> * items = (std::vector<rss_item> *)myfeed;
-	assert (argc == 11);
+	assert (argc == 12);
 	rss_item item(NULL);
 	item.set_guid(argv[0]);
 	item.set_title(argv[1]);
@@ -125,6 +138,7 @@ static int search_item_callback(void * myfeed, int argc, char ** argv, char ** /
 	item.set_enclosure_url(argv[8] ? argv[8] : "");
 	item.set_enclosure_type(argv[9] ? argv[9] : "");
 	item.set_enqueued((std::string("1") == argv[10]));
+	item.set_flags(argv[11] ? argv[11] : "");
 
 	items->push_back(item);
 	return 0;
@@ -170,17 +184,17 @@ void cache::set_pragmas() {
 	
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"setting PRAGMA synchronous = OFF failed");
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	// then we disable case-sensitive matching for the LIKE operator in SQLite, for search operations
 	rc = sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF;", NULL, NULL, NULL);
 
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"setting PRAGMA case_sensitive_like = OFF failed");
+		throw dbexception(db);
 	}
 
-	assert(rc == SQLITE_OK);
 }
 
 void cache::populate_tables() {
@@ -215,6 +229,9 @@ void cache::populate_tables() {
 	rc = sqlite3_exec(db, "ALTER TABLE rss_item ADD enqueued INTEGER(1) NOT NULL DEFAULT 0;", NULL, NULL, NULL);
 	GetLogger().log(LOG_DEBUG, "cache::populate_tables: ALTER TABLE rss_item (3) rc = %d", rc);
 
+	rc = sqlite3_exec(db, "ALTER TABLE rss_item ADD flags VARCHAR(52);", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: ALTER TABLE rss_item (4) rc = %d", rc);
+
 	/* create indexes to speed up certain queries */
 	rc = sqlite3_exec(db, "CREATE INDEX IF NOT EXISTS idx_rssurl ON rss_feed(rssurl);", NULL, NULL, NULL);
 	GetLogger().log(LOG_DEBUG, "cache::populate_tables: CREATE INDEX ON rss_feed(rssurl) (4) rc = %d", rc);
@@ -229,11 +246,31 @@ void cache::populate_tables() {
 		rc = sqlite3_exec(db, "ANALYZE;", NULL, NULL, NULL);
 		GetLogger().log(LOG_DEBUG, "cache::populate_tables: ANALYZE indices (6) rc = %d", rc);
 	}
+
+}
+
+
+
+std::vector<std::string> cache::get_feed_urls() {
+	mtx->lock();
+	std::string query = "SELECT rssurl FROM rss_feed;";
+
+	std::vector<std::string> urls;
+
+	int rc = sqlite3_exec(db, query.c_str(), vectorofstring_callback,&urls, NULL);
+	assert(rc == SQLITE_OK);
+
+	mtx->unlock();
+
+	return urls;
 }
 
 
 // this function writes an rss_feed including all rss_items to the database
 void cache::externalize_rssfeed(rss_feed& feed) {
+	if (feed.rssurl().substr(0,6) == "query:")
+		return;
+
 	mtx->lock();
 	std::ostringstream query;
 	query << "SELECT count(*) FROM rss_feed WHERE rssurl = '" << feed.rssurl() << "';";
@@ -275,6 +312,9 @@ void cache::externalize_rssfeed(rss_feed& feed) {
 // this function reads an rss_feed including all of its rss_items.
 // the feed parameter needs to have the rssurl member set.
 void cache::internalize_rssfeed(rss_feed& feed) {
+	if (feed.rssurl().substr(0,6) == "query:")
+		return;
+
 	mtx->lock();
 
 	/* first, we check whether the feed is there at all */
@@ -284,8 +324,9 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	if (count_cbh.count() == 0) {
 		mtx->unlock();
@@ -298,24 +339,27 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	rc = sqlite3_exec(db,query.c_str(),rssfeed_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	if (feed.items().size() > 0) {
 		feed.items().erase(feed.items().begin(),feed.items().end());
 	}
 
 	/* ...and then the associated items */
-	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued FROM rss_item WHERE feedurl = '%q' ORDER BY pubDate DESC, id DESC;",feed.rssurl().c_str());
+	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE feedurl = '%q' ORDER BY pubDate DESC, id DESC;",feed.rssurl().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	rc = sqlite3_exec(db,query.c_str(),rssitem_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	for (std::vector<rss_item>::iterator it=feed.items().begin(); it != feed.items().end(); ++it) {
 		it->set_cache(this);
+		it->set_feedptr(&feed);
 		it->set_feedurl(feed.rssurl());
 	}
 	
@@ -335,14 +379,15 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 
 void cache::get_latest_items(std::vector<rss_item>& items, unsigned int limit) {
 	mtx->lock();
-	std::string query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued "
+	std::string query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags "
 									"FROM rss_item ORDER BY pubDate DESC, id DESC LIMIT %d;", limit);
 	GetLogger().log(LOG_DEBUG, "running query: %s", query.c_str());
 	int rc = sqlite3_exec(db, query.c_str(), rssitemvector_callback, &items, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	mtx->unlock();
 }
@@ -360,8 +405,9 @@ rss_feed cache::get_feed_by_url(const std::string& feedurl) {
 	rc = sqlite3_exec(db,query.c_str(),rssfeed_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	mtx->unlock();
 
@@ -376,9 +422,9 @@ std::vector<rss_item> cache::search_for_items(const std::string& querystr, const
 	mtx->lock();
 
 	if (feedurl.length() > 0) {
-		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') AND feedurl = '%q' ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str(), feedurl.c_str());
+		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') AND feedurl = '%q' ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str(), feedurl.c_str());
 	} else {
-		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str());
+		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str());
 	}
 
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
@@ -386,8 +432,9 @@ std::vector<rss_item> cache::search_for_items(const std::string& querystr, const
 	rc = sqlite3_exec(db,query.c_str(),search_item_callback,&items,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	mtx->unlock();
 
@@ -400,13 +447,13 @@ void cache::delete_item(const rss_item& item) {
 	int rc = sqlite3_exec(db,query.c_str(),NULL,NULL,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 }
 
 void cache::do_vacuum() {
 	mtx->lock();
-	char * vacuum_query = "VACUUM;";
+	const char * vacuum_query = "VACUUM;";
 	int rc = sqlite3_exec(db,vacuum_query,NULL,NULL,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", vacuum_query, rc);
@@ -456,15 +503,17 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 		rc = sqlite3_exec(db,cleanup_rss_feeds_statement.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", cleanup_rss_feeds_statement.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 
 		GetLogger().log(LOG_DEBUG,"running query: %s", cleanup_rss_items_statement.c_str());
 		rc = sqlite3_exec(db,cleanup_rss_items_statement.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", cleanup_rss_items_statement.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 
 		// rc = sqlite3_exec(db,"VACUUM;",NULL,NULL,NULL);
 		// assert(rc == SQLITE_OK);
@@ -486,36 +535,60 @@ void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 	if (count_cbh.count() > 0) {
-		std::string update = prepare_query("UPDATE rss_item SET title = '%q', author = '%q', url = '%q', feedurl = '%q', content = '%q', enclosure_url = '%q', enclosure_type = '%q' WHERE guid = '%q' AND pubDate != '%u'",
+		std::string update = prepare_query("UPDATE rss_item SET title = '%q', author = '%q', url = '%q', feedurl = '%q', content = '%q', enclosure_url = '%q', enclosure_type = '%q', flags = '%q' WHERE guid = '%q'",
 			item.title_raw().c_str(), item.author_raw().c_str(), item.link().c_str(), 
 			feedurl.c_str(), item.description_raw().c_str(), 
 			item.enclosure_url().c_str(), item.enclosure_type().c_str(),
-			item.guid().c_str(), item.pubDate_timestamp());
+			item.flags().c_str(),
+			item.guid().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", update.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 	} else {
-		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued) "
-								"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
+		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
+								"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
 								item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
 								item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-								item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
+								item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0, item.flags().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", insert.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 	}
 	mtx->unlock();
 }
 
+void cache::catchup_all(rss_feed& feed) {
+	mtx->lock();
+	std::string query = "UPDATE rss_item SET unread = '0' WHERE unread != '0' AND guid IN (";
+
+	for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
+		query.append(prepare_query("'%q',", it->guid().c_str()));
+	}
+	query.append("'');");
+
+	GetLogger().log(LOG_DEBUG, "running query: %s", query.c_str());
+	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
+	}
+
+	mtx->unlock();
+}
+
 /* this function marks all rss_items (optionally of a certain feed url) as read */
 void cache::catchup_all(const std::string& feedurl) {
 	mtx->lock();
@@ -529,8 +602,9 @@ void cache::catchup_all(const std::string& feedurl) {
 	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 	mtx->unlock();
 }
 
@@ -543,8 +617,9 @@ void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
 	}
-	assert(rc == SQLITE_OK);
 
 	if (count_cbh.count() > 0) {
 		std::string update = prepare_query("UPDATE rss_item SET unread = '%d', enqueued = '%d' WHERE guid = '%q'",
@@ -553,20 +628,22 @@ void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", update.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 	} else {
-		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued) "
-										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
+		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
+										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
 										item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
 										item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-										item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
+										item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0, item.flags().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", insert.c_str(), rc);
+			mtx->unlock();
+			throw dbexception(db);
 		}
-		assert(rc == SQLITE_OK);
 	}
 	mtx->unlock();
 }
@@ -583,3 +660,16 @@ std::string cache::prepare_query(const char * format, ...) {
 	}
 	return result;
 }
+
+void cache::update_rssitem_flags(rss_item& item) {
+	mtx->lock();
+	std::string update = prepare_query("UPDATE rss_item SET flags = '%q' WHERE guid = '%q';", item.flags().c_str(), item.guid().c_str());
+	GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
+	int rc = sqlite3_exec(db,update.c_str(), NULL, NULL, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", update.c_str(), rc);
+		mtx->unlock();
+		throw dbexception(db);
+	}
+	mtx->unlock();
+}
