@@ -3,6 +3,7 @@
 #include <logger.h>
 #include <sstream>
 #include <iostream>
+#include <utils.h>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -33,6 +34,17 @@ configcontainer::configcontainer()
 	config_data["proxy"]           = configdata("", configdata::STR);
 	config_data["proxy-auth"]      = configdata("", configdata::STR);
 	config_data["confirm-exit"]    = configdata("no", configdata::BOOL);
+	config_data["error-log"]       = configdata("", configdata::PATH);
+	config_data["notify-screen"]   = configdata("no", configdata::BOOL);
+	config_data["notify-xterm"]    = configdata("no", configdata::BOOL);
+	config_data["notify-program"]  = configdata("", configdata::PATH);
+	config_data["datetime-format"] = configdata("%b %d", configdata::STR);
+	config_data["urls-source"]     = configdata("local", configdata::STR);
+	config_data["bloglines-auth"]  = configdata("", configdata::STR);
+	config_data["bloglines-mark-read"] = configdata("no", configdata::BOOL);
+	config_data["bookmark-cmd"]    = configdata("", configdata::STR);
+	config_data["opml-url"]        = configdata("", configdata::STR);
+	config_data["html-renderer"]   = configdata("internal", configdata::PATH);
 }
 
 configcontainer::~configcontainer()
@@ -81,43 +93,13 @@ action_handler_status configcontainer::handle_action(const std::string& action,
 			return AHS_OK;	
 
 		case configdata::STR:
+		case configdata::PATH:
 			if (params.size() < 1) {
 				return AHS_TOO_FEW_PARAMS;
 			}
 			cfgdata.value = params[0];
 			return AHS_OK;	
 
-		case configdata::PATH: {
-			if (params.size() < 1) {
-				return AHS_TOO_FEW_PARAMS;
-			}
-
-			// a path config option is a bit more difficult to handle, because we need to replace 
-			// a possible "~/" at the beginning of the string with the user's home directory.
-
-			char * homedir;
-			std::string filepath;
-
-			if (!(homedir = ::getenv("HOME"))) {
-				struct passwd * spw = ::getpwuid(::getuid());
-				if (spw) {
-						homedir = spw->pw_dir;
-				} else {
-						homedir = "";
-				}
-			}
-
-			if (strcmp(homedir,"")!=0 && params[0].substr(0,2) == "~/") {
-				filepath.append(homedir);
-				filepath.append(1,'/');
-				filepath.append(params[0].substr(2,params[0].length()-2));
-			} else {
-				filepath.append(params[0]);
-			}
-
-			cfgdata.value = filepath;
-			return AHS_OK;
-		}
 		default:
 			// should not happen
 			return AHS_INVALID_COMMAND;	
@@ -128,7 +110,7 @@ action_handler_status configcontainer::handle_action(const std::string& action,
 }
 
 bool configcontainer::is_bool(const std::string& s) {
-	char * bool_values[] = { "yes", "no", "true", "false", 0 };
+	const char * bool_values[] = { "yes", "no", "true", "false", 0 };
 	for (int i=0; bool_values[i] ; ++i) {
 		if (s == bool_values[i])
 			return true;
@@ -146,7 +128,12 @@ bool configcontainer::is_int(const std::string& s) {
 }
 
 std::string configcontainer::get_configvalue(const std::string& key) {
-	return config_data[key].value;
+	std::string retval = config_data[key].value;
+	if (config_data[key].type == configdata::PATH) {
+		retval = utils::resolve_tilde(retval);
+	}
+
+	return retval;
 }
 
 int configcontainer::get_configvalue_as_int(const std::string& key) {
