@@ -12,16 +12,22 @@
 #include <cstdlib>
 #include <iostream>
 #include <fstream>
+#include <cerrno>
 
 #include <sys/time.h>
 #include <ctime>
+#include <cassert>
 #include <signal.h>
+#include <sys/utsname.h>
 
 #include <nxml.h>
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <pwd.h>
 
+#include <ncursesw/ncurses.h>
+
 #include <config.h>
 
 using namespace newsbeuter;
@@ -42,6 +48,12 @@ void ignore_signal(int sig) {
 	GetLogger().log(LOG_WARN, "caught signal %d but ignored it", sig);
 }
 
+void omg_a_child_died(int /* sig */) {
+	pid_t pid;
+	int stat;
+	while ((pid = waitpid(-1,&stat,WNOHANG)) > 0);
+}
+
 controller::controller() : v(0), rsscache(0), url_file("urls"), cache_file("cache.db"), config_file("config"), queue_file("queue"), refresh_on_start(false), cfg(0) {
 	std::ostringstream cfgfile;
 
@@ -89,16 +101,18 @@ void controller::run(int argc, char * argv[]) {
 
 	::signal(SIGINT, ctrl_c_action);
 #ifndef DEBUG
-	::signal(SIGSEGV, ctrl_c_action);
+	// ::signal(SIGSEGV, ctrl_c_action);
 #endif
 	::signal(SIGPIPE, ignore_signal);
 	::signal(SIGHUP, ctrl_c_action);
+	::signal(SIGCHLD, omg_a_child_died);
 
 	bool do_import = false, do_export = false, cachefile_given_on_cmdline = false, do_vacuum = false;
+	bool offline_mode = false, real_offline_mode = false;
 	std::string importfile;
 
 	do {
-		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:v"))<0)
+		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:vVo"))<0)
 			continue;
 		switch (c) {
 			case ':': /* fall-through */
@@ -135,6 +149,12 @@ void controller::run(int argc, char * argv[]) {
 			case 'v':
 				do_vacuum = true;
 				break;
+			case 'V':
+				version_information();
+				break;
+			case 'o':
+				offline_mode = true;
+				break;
 			case 'd': // this is an undocumented debug commandline option!
 				GetLogger().set_logfile(optarg);
 				break;
@@ -153,20 +173,14 @@ void controller::run(int argc, char * argv[]) {
 		}
 	} while (c != -1);
 
-	urlcfg.load_config(url_file);
 
 	if (do_import) {
 		GetLogger().log(LOG_INFO,"Importing OPML file from %s",importfile.c_str());
+		urlcfg = new file_urlreader(url_file);
 		import_opml(importfile.c_str());
 		return;
 	}
 
-	if (urlcfg.get_urls().size() == 0) {
-		GetLogger().log(LOG_ERROR,"no URLs configured.");
-		snprintf(msgbuf, sizeof(msgbuf), _("Error: no URLs configured. Please fill the file %s with RSS feed URLs or import an OPML file."), url_file.c_str());
-		std::cout << msgbuf << std::endl << std::endl;
-		usage(argv[0]);
-	}
 
 	if (!do_export) {
 		snprintf(msgbuf, sizeof(msgbuf), _("Starting %s %s..."), PROGRAM_NAME, PROGRAM_VERSION);
@@ -174,7 +188,11 @@ void controller::run(int argc, char * argv[]) {
 
 		pid_t pid;
 		if (!utils::try_fs_lock(lock_file, pid)) {
-			GetLogger().log(LOG_ERROR,"an instance is alredy running: pid = %u",pid);
+			if (pid > 0) {
+				GetLogger().log(LOG_ERROR,"an instance is already running: pid = %u",pid);
+			} else {
+				GetLogger().log(LOG_ERROR,"something went wrong with the lock: %s", strerror(errno));
+			}
 			snprintf(msgbuf, sizeof(msgbuf), _("Error: an instance of %s is already running (PID: %u)"), PROGRAM_NAME, pid);
 			std::cout << msgbuf << std::endl;
 			return;
@@ -191,10 +209,14 @@ void controller::run(int argc, char * argv[]) {
 	colormanager * colorman = new colormanager();
 	colorman->register_commands(cfgparser);
 
-	keymap keys;
+	keymap keys(KM_NEWSBEUTER);
 	cfgparser.register_handler("bind-key",&keys);
 	cfgparser.register_handler("unbind-key",&keys);
 
+	cfgparser.register_handler("ignore-article",&ign);
+
+	cfgparser.register_handler("define-filter",&filters);
+
 	try {
 		cfgparser.parse("/etc/" PROGRAM_NAME "/config");
 		cfgparser.parse(config_file);
@@ -205,25 +227,91 @@ void controller::run(int argc, char * argv[]) {
 		return;	
 	}
 
-	if (colorman->colors_loaded())
-		colorman->set_colors(v);
+	if (colorman->colors_loaded()) {
+		v->set_colors(*colorman);
+	}
 	delete colorman;
-	
+
+	if (cfg->get_configvalue("error-log").length() > 0) {
+		GetLogger().set_errorlogfile(cfg->get_configvalue("error-log").c_str());
+	}
+
 	if (!do_export)
 		std::cout << _("done.") << std::endl;
 
+	// create cache object
+	std::string cachefilepath = cfg->get_configvalue("cache-file");
+	if (cachefilepath.length() > 0 && !cachefile_given_on_cmdline) {
+		cache_file = cachefilepath.c_str();
+	}
+
+	if (!do_export) {
+		std::cout << _("Opening cache...");
+		std::cout.flush();
+	}
+	rsscache = new cache(cache_file,cfg);
+	if (!do_export) {
+		std::cout << _("done.") << std::endl;
+	}
+
+
+	std::string type = cfg->get_configvalue("urls-source");
+	if (type == "local") {
+		urlcfg = new file_urlreader(url_file);
+	} else if (type == "bloglines") {
+		urlcfg = new bloglines_urlreader(cfg);
+		real_offline_mode = offline_mode;
+	} else if (type == "opml") {
+		urlcfg = new opml_urlreader(cfg);
+		real_offline_mode = offline_mode;
+	} else {
+		GetLogger().log(LOG_ERROR,"unknown urls-source `%s'", urlcfg->get_source().c_str());
+	}
+
+	if (real_offline_mode) {
+		if (!do_export) {
+			snprintf(msgbuf,sizeof(msgbuf), _("Loading URLs from local cache..."));
+			std::cout << msgbuf;
+			std::cout.flush();
+		}
+		urlcfg->set_offline(true);
+		urlcfg->get_urls() = rsscache->get_feed_urls();
+		if (!do_export) {
+			std::cout << _("done.") << std::endl;
+		}
+	} else {
+		if (!do_export) {
+			snprintf(msgbuf,sizeof(msgbuf), _("Loading URLs from %s..."), urlcfg->get_source().c_str());
+			std::cout << msgbuf;
+			std::cout.flush();
+		}
+		urlcfg->reload();
+		if (!do_export) {
+			std::cout << _("done.") << std::endl;
+		}
+	}
+
+	if (urlcfg->get_urls().size() == 0) {
+		GetLogger().log(LOG_ERROR,"no URLs configured.");
+		if (type == "local") {
+			snprintf(msgbuf, sizeof(msgbuf), _("Error: no URLs configured. Please fill the file %s with RSS feed URLs or import an OPML file."), url_file.c_str());
+		} else if (type == "bloglines") {
+			snprintf(msgbuf, sizeof(msgbuf), _("It looks like you haven't configured any feeds in your bloglines account. Please do so, and try again."));
+		} else if (type == "opml") {
+			snprintf(msgbuf, sizeof(msgbuf), _("It looks like the OPML feed you subscribed contains no feeds. Please fill it with feeds, and try again."));
+		} else {
+			assert(0); // shouldn't happen
+		}
+		std::cout << msgbuf << std::endl << std::endl;
+		usage(argv[0]);
+	}
+
 	if (!do_export && !do_vacuum)
 		std::cout << _("Loading articles from cache...");
 	if (do_vacuum)
 		std::cout << _("Opening cache...");
 	std::cout.flush();
 
-	std::string cachefilepath = cfg->get_configvalue("cache-file");
-	if (cachefilepath.length() > 0 && !cachefile_given_on_cmdline) {
-		cache_file = cachefilepath.c_str();
-	}
-
-	rsscache = new cache(cache_file,cfg);
 
 	if (do_vacuum) {
 		std::cout << _("done.") << std::endl;
@@ -235,15 +323,21 @@ void controller::run(int argc, char * argv[]) {
 		return;
 	}
 
-	for (std::vector<std::string>::const_iterator it=urlcfg.get_urls().begin(); it != urlcfg.get_urls().end(); ++it) {
+	for (std::vector<std::string>::const_iterator it=urlcfg->get_urls().begin(); it != urlcfg->get_urls().end(); ++it) {
 		rss_feed feed(rsscache);
 		feed.set_rssurl(*it);
-		feed.set_tags(urlcfg.get_tags(*it));
-		rsscache->internalize_rssfeed(feed);
+		feed.set_tags(urlcfg->get_tags(*it));
+		try {
+			rsscache->internalize_rssfeed(feed);
+		} catch(const dbexception& e) {
+			std::cout << _("Error while loading feeds from database: ") << e.what() << std::endl;
+			utils::remove_fs_lock(lock_file);
+			return;
+		}
 		feeds.push_back(feed);
 	}
 
-	std::vector<std::string> tags = urlcfg.get_alltags();
+	std::vector<std::string> tags = urlcfg->get_alltags();
 
 	if (!do_export)
 		std::cout << _("done.") << std::endl;
@@ -271,8 +365,12 @@ void controller::run(int argc, char * argv[]) {
 
 	std::cout << _("Cleaning up cache...");
 	std::cout.flush();
-	rsscache->cleanup_cache(feeds);
-	std::cout << _("done.") << std::endl;
+	try {
+		rsscache->cleanup_cache(feeds);
+		std::cout << _("done.") << std::endl;
+	} catch (const dbexception& e) {
+		std::cout << _("failed: ") << e.what() << std::endl;
+	}
 
 	utils::remove_fs_lock(lock_file);
 }
@@ -282,7 +380,14 @@ void controller::update_feedlist() {
 }
 
 void controller::catchup_all() {
-	rsscache->catchup_all();
+	try {
+		rsscache->catchup_all();
+	} catch (const dbexception& e) {
+		char buf[1024];
+		snprintf(buf, sizeof(buf), _("Error: couldn't mark all feeds read: %s"), e.what());
+		v->show_error(buf);
+		return;
+	}
 	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
 		if (it->items().size() > 0) {
 			for (std::vector<rss_item>::iterator jt=it->items().begin();jt!=it->items().end();++jt) {
@@ -295,10 +400,14 @@ void controller::catchup_all() {
 void controller::mark_all_read(unsigned int pos) {
 	if (pos < feeds.size()) {
 		rss_feed& feed = feeds[pos];
-		rsscache->catchup_all(feed.rssurl());
+		if (feed.rssurl().substr(0,6) == "query:") {
+			rsscache->catchup_all(feed);
+		} else {
+			rsscache->catchup_all(feed.rssurl());
+		}
 		if (feed.items().size() > 0) {
 			for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-				it->set_unread_nowrite(false);
+				it->set_unread_nowrite_notify(false);
 			}
 		}
 	}
@@ -326,19 +435,24 @@ void controller::reload(unsigned int pos, unsigned int max) {
 		v->set_status(msgbuf);
 		GetLogger().log(LOG_DEBUG, "controller::reload: after setting status");
 				
-		rss_parser parser(feed.rssurl().c_str(), rsscache, cfg);
+		rss_parser parser(feed.rssurl().c_str(), rsscache, cfg, &ign);
 		GetLogger().log(LOG_DEBUG, "controller::reload: created parser");
 		try {
 			feed = parser.parse();
 			GetLogger().log(LOG_DEBUG, "controller::reload: after parser.parse");
-			
-			rsscache->externalize_rssfeed(feed);
-			GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
+			if (!feed.is_empty()) {
+				GetLogger().log(LOG_DEBUG, "controller::reload: feed is nonempty, saving");
+				rsscache->externalize_rssfeed(feed);
+				GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
 
-			rsscache->internalize_rssfeed(feed);
-			GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
-			feed.set_tags(urlcfg.get_tags(feed.rssurl()));
-			feeds[pos] = feed;
+				rsscache->internalize_rssfeed(feed);
+				GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
+				feed.set_tags(urlcfg->get_tags(feed.rssurl()));
+				feeds[pos] = feed;
+				v->notify_itemlist_change(feed);
+			} else {
+				GetLogger().log(LOG_DEBUG, "controller::reload: feed is empty, not saving");
+			}
 
 			for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
 				if (cfg->get_configvalue_as_bool("podcast-auto-enqueue") && !it->enqueued() && it->enclosure_url().length() > 0) {
@@ -356,6 +470,10 @@ void controller::reload(unsigned int pos, unsigned int max) {
 			v->set_feedlist(feeds);
 			GetLogger().log(LOG_DEBUG, "controller::reload: after set_feedlist");
 			v->set_status("");
+		} catch (const dbexception& e) {
+			char buf[1024];
+			snprintf(buf, sizeof(buf), _("Error while retrieving %s: %s"), feed.rssurl().c_str(), e.what());
+			v->set_status(buf);
 		} catch (const std::string& errmsg) {
 			char buf[1024];
 			snprintf(buf, sizeof(buf), _("Error while retrieving %s: %s"), feed.rssurl().c_str(), errmsg.c_str());
@@ -375,10 +493,68 @@ rss_feed * controller::get_feed(unsigned int pos) {
 
 void controller::reload_all() {
 	GetLogger().log(LOG_DEBUG,"controller::reload_all: starting with reload all...");
+	unsigned int unread_feeds, unread_articles;
+	compute_unread_numbers(unread_feeds, unread_articles);
+	time_t t1, t2, dt;
+	t1 = time(NULL);
 	for (unsigned int i=0;i<feeds.size();++i) {
 		GetLogger().log(LOG_DEBUG, "controller::reload_all: reloading feed #%u", i);
 		this->reload(i,feeds.size());
 	}
+	t2 = time(NULL);
+	dt = t2 - t1;
+	GetLogger().log(LOG_INFO, "controller::reload_all: reload took %d seconds", dt);
+
+	unsigned int unread_feeds2, unread_articles2;
+	compute_unread_numbers(unread_feeds2, unread_articles2);
+	if (unread_feeds2 != unread_feeds || unread_articles2 != unread_articles) {
+		char buf[2048];
+		snprintf(buf,sizeof(buf),_("newsbeuter: finished reload, %u unread feeds (%u unread articles total)"), unread_feeds2, unread_articles2);
+		this->notify(buf);
+	}
+}
+
+void controller::notify(const std::string& msg) {
+	if (cfg->get_configvalue_as_bool("notify-screen")) {
+		GetLogger().log(LOG_DEBUG, "controller:notify: notifying screen");
+		std::cout << "\033^" << msg << "\033\\";
+		std::cout.flush();
+	}
+	if (cfg->get_configvalue_as_bool("notify-xterm")) {
+		GetLogger().log(LOG_DEBUG, "controller:notify: notifying xterm");
+		std::cout << "\033]2;" << msg << "\033\\";
+		std::cout.flush();
+	}
+	if (cfg->get_configvalue("notify-program").length() > 0) {
+		std::string prog = cfg->get_configvalue("notify-program");
+		GetLogger().log(LOG_DEBUG, "controller:notify: notifying external program `%s'", prog.c_str());
+		utils::run_command(prog, msg);
+	}
+	/* // TODO: implement Growl support
+	if (cfg->get_configvalue_as_bool("notify-growl")) {
+		std::vector<std::string> tokens = utils::tokenize(cfg->get_configvalue("growl-config"), ":");
+		if (tokens.size() >= 1) {
+			std::string hostname = tokens[0];
+			std::string password;
+			if (tokens.size() >= 2) {
+				password = tokens[1];
+			}
+			growlnotifier->send_notify(hostname, password, "newsbeuter", msg);
+		}
+	}
+	*/
+}
+
+void controller::compute_unread_numbers(unsigned int& unread_feeds, unsigned int& unread_articles) {
+	unread_feeds = 0;
+	unread_articles = 0;
+	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
+		unsigned int items = it->unread_item_count();
+		if (items > 0) {
+			++unread_feeds;
+			unread_articles += items;
+		}
+	}
 }
 
 bool controller::trylock_reload_mutex() {
@@ -396,6 +572,24 @@ void controller::start_reload_all_thread() {
 	dlt->start();
 }
 
+void controller::version_information() {
+	std::cout << PROGRAM_NAME << " " << PROGRAM_VERSION << " - " << PROGRAM_URL << std::endl;
+	std::cout << "Copyright (C) 2006-2007 Andreas Krennmair" << std::endl << std::endl;
+
+	struct utsname xuts;
+	uname(&xuts);
+
+	std::cout << "System: " << xuts.sysname << " " << xuts.release << " (" << xuts.machine << ")" << std::endl;
+#if defined(__GNUC__) && defined(__VERSION__)
+	std::cout << "Compiler: g++ " << __VERSION__ << std::endl;
+#endif
+	std::cout << "ncurses: " << curses_version() << " (compiled with " << NCURSES_VERSION << ")" << std::endl;
+	std::cout << "libcurl: " << curl_version()  << " (compiled with " << LIBCURL_VERSION << ")" << std::endl;
+	std::cout << "SQLite: " << sqlite3_libversion() << " (compiled with " << SQLITE_VERSION << ")" << std::endl;
+
+	::exit(EXIT_SUCCESS);
+}
+
 void controller::usage(char * argv0) {
 	char buf[2048];
 	snprintf(buf, sizeof(buf), 
@@ -407,6 +601,8 @@ void controller::usage(char * argv0) {
 				"-c <cachefile>  use <cachefile> as cache file\n"
 				"-C <configfile> read configuration from <configfile>\n"
 				"-v              clean up cache thoroughly\n"
+				"-o              activate offline mode (only applies to bloglines synchronization mode)\n"
+				"-V              get version information\n"
 				"-h              this help\n"), PROGRAM_NAME, PROGRAM_VERSION, argv0);
 	std::cout << buf;
 	::exit(EXIT_FAILURE);
@@ -419,13 +615,13 @@ void controller::import_opml(const char * filename) {
 
 	ret = nxml_new (&data);
 	if (ret != NXML_OK) {
-		puts (nxml_strerror (ret));
+		puts (nxml_strerror (data, ret));
 		return;
 	}
 
 	ret = nxml_parse_file (data, const_cast<char *>(filename));
 	if (ret != NXML_OK) {
-		puts (nxml_strerror (ret));
+		puts (nxml_strerror (data, ret));
 		return;
 	}
 
@@ -434,8 +630,9 @@ void controller::import_opml(const char * filename) {
 	if (root) {
 		body = nxmle_find_element(data, root, "body", NULL);
 		if (body) {
+			GetLogger().log(LOG_DEBUG, "import_opml: found body");
 			rec_find_rss_outlines(body, "");
-			urlcfg.write_config();
+			urlcfg->write_config();
 		}
 	}
 
@@ -451,7 +648,16 @@ void controller::export_opml() {
 	std::cout << "\t<head>" << std::endl << "\t\t<title>" PROGRAM_NAME " - Exported Feeds</title>" << std::endl << "\t</head>" << std::endl;
 	std::cout << "\t<body>" << std::endl;
 	for (std::vector<rss_feed>::iterator it=feeds.begin(); it != feeds.end(); ++it) {
-		std::cout << "\t\t<outline type=\"rss\" xmlUrl=\"" << it->rssurl() << "\" title=\"" << it->title() << "\" />" << std::endl;
+		if (it->rssurl().substr(0,6) != "query:" && it->rssurl().substr(0,7) != "filter:") {
+			std::string rssurl = utils::replace_all(it->rssurl(),"&","&amp;");
+			std::string link = utils::replace_all(it->link(),"&", "&amp;");
+			std::string title = utils::replace_all(it->title(),"&", "&amp;");
+
+			rssurl = utils::replace_all(rssurl, "\"", "&quot;");
+			link = utils::replace_all(link, "\"", "&quot;");
+			title = utils::replace_all(title, "\"", "&quot;");
+			std::cout << "\t\t<outline type=\"rss\" xmlUrl=\"" << rssurl << "\" htmlUrl=\"" << link << "\" title=\"" << title << "\" />" << std::endl;
+		}
 	}
 	std::cout << "\t</body>" << std::endl;
 	std::cout << "</opml>" << std::endl;
@@ -460,7 +666,6 @@ void controller::export_opml() {
 void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 	while (node) {
 		char * url = nxmle_find_attribute(node, "xmlUrl", NULL);
-		char * type = nxmle_find_attribute(node, "type", NULL);
 		std::string newtag = tag;
 
 		if (!url) {
@@ -468,32 +673,35 @@ void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 		}
 
 		if (node->type == NXML_TYPE_ELEMENT && strcmp(node->value,"outline")==0) {
-			if (type && (strcmp(type,"rss")==0 || strcmp(type,"link")==0)) {
-				if (url) {
+			if (url) {
 
-					GetLogger().log(LOG_DEBUG,"OPML import: found RSS outline with url = %s",url);
+				GetLogger().log(LOG_DEBUG,"OPML import: found RSS outline with url = %s",url);
 
-					bool found = false;
+				bool found = false;
 
-					for (std::vector<std::string>::iterator it = urlcfg.get_urls().begin(); it != urlcfg.get_urls().end(); ++it) {
+				GetLogger().log(LOG_DEBUG, "OPML import: size = %u", urlcfg->get_urls().size());
+				if (urlcfg->get_urls().size() > 0) {
+					for (std::vector<std::string>::iterator it = urlcfg->get_urls().begin(); it != urlcfg->get_urls().end(); ++it) {
 						if (*it == url) {
 							found = true;
 						}
 					}
+				}
 
-					if (!found) {
-						GetLogger().log(LOG_DEBUG,"OPML import: added url = %s",url);
-						urlcfg.get_urls().push_back(std::string(url));
-						if (tag.length() > 0) {
-							GetLogger().log(LOG_DEBUG, "OPML import: appending tag %s to url %s", tag.c_str(), url);
-							urlcfg.get_tags(url).push_back(tag);
-						}
-					} else {
-						GetLogger().log(LOG_DEBUG,"OPML import: url = %s is already in list",url);
+				if (!found) {
+					GetLogger().log(LOG_DEBUG,"OPML import: added url = %s",url);
+					urlcfg->get_urls().push_back(std::string(url));
+					if (tag.length() > 0) {
+						GetLogger().log(LOG_DEBUG, "OPML import: appending tag %s to url %s", tag.c_str(), url);
+						urlcfg->get_tags(url).push_back(tag);
 					}
+				} else {
+					GetLogger().log(LOG_DEBUG,"OPML import: url = %s is already in list",url);
 				}
 			} else {
 				char * text = nxmle_find_attribute(node, "text", NULL);
+				if (!text)
+					text = nxmle_find_attribute(node, "title", NULL);
 				if (text) {
 					if (newtag.length() > 0) {
 						newtag.append("/");
@@ -511,7 +719,12 @@ void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 
 
 std::vector<rss_item> controller::search_for_items(const std::string& query, const std::string& feedurl) {
-	return rsscache->search_for_items(query, feedurl);
+	std::vector<rss_item> items = rsscache->search_for_items(query, feedurl);
+	GetLogger().log(LOG_DEBUG, "controller::search_for_items: setting feed pointers");
+	for (std::vector<rss_item>::iterator it=items.begin();it!=items.end();++it) {
+		it->set_feedptr(get_feed_by_url(it->feedurl()));
+	}
+	return items;
 }
 
 rss_feed * controller::get_feed_by_url(const std::string& feedurl) {
@@ -551,3 +764,59 @@ void controller::enqueue_url(const std::string& url) {
 	}
 }
 
+void controller::reload_urls_file() {
+	urlcfg->reload();
+	std::vector<rss_feed> new_feeds;
+
+	for (std::vector<std::string>::const_iterator it=urlcfg->get_urls().begin();it!=urlcfg->get_urls().end();++it) {
+		bool found = false;
+		for (std::vector<rss_feed>::iterator jt=feeds.begin();jt!=feeds.end();++jt) {
+			if (*it == jt->rssurl()) {
+				found = true;
+				new_feeds.push_back(*jt);
+				break;
+			}
+		}
+		if (!found) {
+			rss_feed new_feed(rsscache);
+			new_feed.set_rssurl(*it);
+			new_feed.set_tags(urlcfg->get_tags(*it));
+			try {
+				rsscache->internalize_rssfeed(new_feed);
+			} catch(const dbexception& e) {
+				throw e; // TODO ?
+			}
+			new_feeds.push_back(new_feed);
+		}
+	}
+
+	feeds = new_feeds;
+
+	update_feedlist();
+}
+
+void controller::set_feedptrs(rss_feed& feed) {
+	for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
+		it->set_feedptr(&feed);
+	}
+}
+
+std::string controller::bookmark(const std::string& url, const std::string& title, const std::string& description) {
+	std::string bookmark_cmd = cfg->get_configvalue("bookmark-cmd");
+	if (bookmark_cmd.length() > 0) {
+		char * my_argv[4];
+		my_argv[0] = "/bin/sh";
+		my_argv[1] = "-c";
+
+		// wow. what an abuse.
+		std::string cmdline = bookmark_cmd + " " + stfl::quote(url) + " " + stfl::quote(title) + " " + stfl::quote(description);
+
+		my_argv[2] = const_cast<char *>(cmdline.c_str());
+		my_argv[3] = NULL;
+
+		return utils::run_program(my_argv, "");
+	} else {
+		return _("bookmarking support is not configured. Please set the configuration variable `bookmark-cmd' accordingly.");
+	}
+}
+
