@@ -8,14 +8,17 @@ namespace newsbeuter {
 	
 struct op_desc {
 	operation op;
-	char * opstr;
-	char * default_key;
-	char * help_text;
+	const char * opstr;
+	const char * default_key;
+	const char * help_text;
 	unsigned short flags;
 };
 
+/*
+ * This is the list of operations, defining operation, operation name (for keybindings), default key, description, and where it's valid
+ */
 static op_desc opdescs[] = {
-	{ OP_OPEN, "open", "enter", _("Open feed/article"), KM_NEWSBEUTER },
+	{ OP_OPEN, "open", "ENTER", _("Open feed/article"), KM_NEWSBEUTER },
 	{ OP_QUIT, "quit", "q", _("Return to previous dialog/Quit"), KM_BOTH },
 	{ OP_RELOAD, "reload", "r", _("Reload currently selected feed"), KM_NEWSBEUTER },
 	{ OP_RELOADALL, "reload-all", "R", _("Reload all feeds"), KM_NEWSBEUTER },
@@ -23,16 +26,18 @@ static op_desc opdescs[] = {
 	{ OP_MARKALLFEEDSREAD, "mark-all-feeds-read", "C", _("Mark all feeds read"), KM_NEWSBEUTER },
 	{ OP_SAVE, "save", "s", _("Save article"), KM_NEWSBEUTER },
 	{ OP_NEXTUNREAD, "next-unread", "n", _("Go to next unread article"), KM_NEWSBEUTER },
+	{ OP_PREVUNREAD, "prev-unread", "p", _("Go to previous unread article"), KM_NEWSBEUTER },
 	{ OP_OPENINBROWSER, "open-in-browser", "o", _("Open article in browser"), KM_NEWSBEUTER },
 	{ OP_HELP, "help", "?", _("Open help dialog"), KM_BOTH },
-	{ OP_TOGGLESOURCEVIEW, "toggle-source-view", "^u", _("Toggle source view"), KM_NEWSBEUTER },
+	{ OP_TOGGLESOURCEVIEW, "toggle-source-view", "^U", _("Toggle source view"), KM_NEWSBEUTER },
 	{ OP_TOGGLEITEMREAD, "toggle-article-read", "N", _("Toggle read status for article"), KM_NEWSBEUTER },
 	{ OP_TOGGLESHOWREAD, "toggle-show-read-feeds", "l", _("Toggle show read feeds"), KM_NEWSBEUTER },
 	{ OP_SHOWURLS, "show-urls", "u", _("Show URLs in current article"), KM_NEWSBEUTER },
-	{ OP_CLEARTAG, "clear-tag", "^t", _("Clear current tag"), KM_NEWSBEUTER },
+	{ OP_CLEARTAG, "clear-tag", "^T", _("Clear current tag"), KM_NEWSBEUTER },
 	{ OP_SETTAG, "set-tag", "t", _("Select tag"), KM_NEWSBEUTER },
 	{ OP_SEARCH, "open-search", "/", _("Open search dialog"), KM_NEWSBEUTER },
 	{ OP_ENQUEUE, "enqueue", "e", _("Add download to queue"), KM_NEWSBEUTER },
+	{ OP_RELOADURLS, "reload-urls", "^R", _("Reload the list of URLs from the configuration"), KM_NEWSBEUTER },
 	{ OP_PB_DOWNLOAD, "pb-download", "d", _("Download file"), KM_PODBEUTER },
 	{ OP_PB_CANCEL, "pb-cancel", "c", _("Cancel download"), KM_PODBEUTER },
 	{ OP_PB_DELETE, "pb-delete", "D", _("Mark download as deleted"), KM_PODBEUTER },
@@ -41,30 +46,64 @@ static op_desc opdescs[] = {
 	{ OP_PB_PLAY, "pb-play", "p", _("Start player with currently selected download"), KM_PODBEUTER },
 	{ OP_PB_MOREDL, "pb-increase-max-dls", "+", _("Increase the number of concurrent downloads"), KM_PODBEUTER },
 	{ OP_PB_LESSDL, "pb-decreate-max-dls", "-", _("Decrease the number of concurrent downloads"), KM_PODBEUTER },
-	{ OP_REDRAW, "redraw", "^l", _("Redraw screen"), KM_BOTH },
+	{ OP_REDRAW, "redraw", "^L", _("Redraw screen"), KM_BOTH },
+	{ OP_CMDLINE, "cmdline", ":", _("Open the commandline"), KM_NEWSBEUTER },
+	{ OP_SETFILTER, "set-filter", "F", _("Set a filter"), KM_NEWSBEUTER },
+	{ OP_SELECTFILTER, "select-filter", "f", _("Select a predefined filter"), KM_NEWSBEUTER },
+	{ OP_CLEARFILTER, "clear-filter", "^F", _("Clear currently set filter"), KM_NEWSBEUTER },
+	{ OP_BOOKMARK, "bookmark", "^B", _("Bookmark current link/article"), KM_NEWSBEUTER },
+	{ OP_EDITFLAGS, "edit-flags", "^E", _("Edit flags"), KM_NEWSBEUTER },
+	{ OP_NEXTFEED, "next-unread-feed", "^N", "Go to next unread feed", KM_NEWSBEUTER },
+	{ OP_PREVFEED, "prev-unread-feed", "^P", "Go to previous unread feed", KM_NEWSBEUTER },
+
+	{ OP_SK_UP, "up", "UP", NULL, KM_SYSKEYS },
+	{ OP_SK_DOWN, "down", "DOWN", NULL, KM_SYSKEYS },
+	{ OP_SK_PGUP, "pageup", "PAGEUP", NULL, KM_SYSKEYS },
+	{ OP_SK_PGDOWN, "pagedown", "PAGEDOWN", NULL, KM_SYSKEYS },
+
+	{ OP_INT_END_QUESTION, "XXXNOKEY-end-question", "end-question", NULL, KM_INTERNAL },
+	{ OP_INT_CANCEL_QNA, "XXXNOKEY-cancel-qna", "cancel-qna", NULL, KM_INTERNAL },
+	{ OP_INT_QNA_NEXTHIST, "XXXNOKEY-qna-next-history", "qna-next-history", NULL, KM_INTERNAL },
+	{ OP_INT_QNA_PREVHIST, "XXXNOKEY-qna-prev-history", "qna-prev-history", NULL, KM_INTERNAL },
+
 	{ OP_NIL, NULL, NULL, NULL, 0 }
 };
 
-keymap::keymap() { 
-	for (int i=0;opdescs[i].help_text;++i) {
-		keymap_[opdescs[i].default_key] = opdescs[i].op;
+keymap::keymap(unsigned flags) { 
+	/*
+	 * At startup, initialize the keymap with the default settings from the list above.
+	 */
+	for (int i=0;opdescs[i].op != OP_NIL;++i) {
+		if (opdescs[i].flags & (flags | KM_INTERNAL | KM_SYSKEYS)) {
+			keymap_[opdescs[i].default_key] = opdescs[i].op;
+		}
 	}
 }
 
-void keymap::get_keymap_descriptions(std::vector<std::pair<std::string,std::string> >& descs, unsigned short flags) {
+void keymap::get_keymap_descriptions(std::vector<keymap_desc>& descs, unsigned short flags) {
+	/*
+	 * Here we return the keymap descriptions for the specified application (handed to us via flags)
+	 * This is used for the help screen.
+	 */
 	for (std::map<std::string,operation>::iterator it=keymap_.begin();it!=keymap_.end();++it) {
 		operation op = it->second;
 		if (op != OP_NIL) {
 			std::string helptext;
+			std::string opname;
 			bool add = false;
 			for (int i=0;opdescs[i].help_text;++i) {
 				if (opdescs[i].op == op && opdescs[i].flags & flags) {
 					helptext = gettext(opdescs[i].help_text);
+					opname = opdescs[i].opstr;
 					add = true;
 				}
 			}
 			if (add) {
-				descs.push_back(std::pair<std::string,std::string>(it->first, helptext));
+				keymap_desc desc;
+				desc.key = it->first;
+				desc.cmd = opname;
+				desc.desc = helptext;
+				descs.push_back(desc);
 			}
 		}
 	}
@@ -93,14 +132,15 @@ operation keymap::get_opcode(const std::string& opstr) {
 }
 
 char keymap::get_key(const std::string& keycode) {
-	if (strncmp(keycode.c_str(),"CHAR(",5)==0) {
-		unsigned int x;
-		char c;
-		sscanf(keycode.c_str(),"CHAR(%u)",&x);
-		if (x >= 32 && x <= 126) {
-			c = static_cast<char>(x);
-			return c;
-		}
+	if (keycode == "ENTER") {
+		return '\n';
+	} else if (keycode == "ESC") {
+		return 27;
+	} else if (keycode[0] == '^') {
+		char chr = keycode[1];
+		return chr - '@';
+	} else { // TODO: implement more keys
+		return keycode[0];
 	}
 	return 0;
 }
@@ -108,30 +148,7 @@ char keymap::get_key(const std::string& keycode) {
 operation keymap::get_operation(const std::string& keycode) {
 	std::string key;
 	if (keycode.length() > 0) {
-		if (keycode == "ENTER") {
-			key = "enter";
-		} else if (keycode == "ESC") {
-			key = "esc";
-		} else if (keycode[0] == 'F') {
-			key = keycode;
-			key[0] = 'f';
-		} else if (strncmp(keycode.c_str(),"CHAR(",5)==0) {
-			unsigned int x;
-			char c;
-			sscanf(keycode.c_str(),"CHAR(%u)",&x);
-			// std::cerr << x << std::endl;
-			if (32 == x) {
-				key.append("space");
-			} else if (x > 32 && x <= 126) {
-				c = static_cast<char>(x);
-				key.append(1,c);
-			} else if (x<=26) {
-				key.append("^");
-				key.append(1,static_cast<char>(0x60 + x));
-			} else {
-				// TODO: handle special keys
-			}
-		}
+		key = keycode;
 	} else {
 		key = "NIL";
 	}
@@ -139,11 +156,18 @@ operation keymap::get_operation(const std::string& keycode) {
 }
 
 action_handler_status keymap::handle_action(const std::string& action, const std::vector<std::string>& params) {
+	/*
+	 * The keymap acts as config_action_handler so that all the key-related configuration is immediately
+	 * handed to it.
+	 */
 	GetLogger().log(LOG_DEBUG,"keymap::handle_action(%s, ...) called",action.c_str());
 	if (action == "bind-key") {
 		if (params.size() < 2) {
 			return AHS_TOO_FEW_PARAMS;
 		} else {
+			operation op = get_opcode(params[1]);
+			if (op > OP_SK_MIN && op < OP_SK_MAX)
+				unset_key(getkey(get_opcode(params[1])));
 			set_key(get_opcode(params[1]), params[0]);
 			// keymap_[params[0]] = get_opcode(params[1]);
 			return AHS_OK;
@@ -162,7 +186,7 @@ action_handler_status keymap::handle_action(const std::string& action, const std
 std::string keymap::getkey(operation op) {
 	for (std::map<std::string,operation>::iterator it=keymap_.begin(); it!=keymap_.end(); ++it) {
 		if (it->second == op)
-			return it->first;	
+			return it->first;
 	}	
 	return "<none>";
 }
