@@ -23,6 +23,12 @@ void reloadthread::run() {
 			}
 		}
 
-		::sleep((oldtime + waittime_sec) - time(NULL));
+		time_t seconds_to_wait = 0;
+		if (oldtime + waittime_sec > time(NULL))
+			seconds_to_wait = oldtime + waittime_sec - time(NULL);
+
+		while (seconds_to_wait > 0) {
+			seconds_to_wait = ::sleep(seconds_to_wait);
+		}
 	}
 }
