@@ -6,12 +6,37 @@
 #include <fcntl.h>
 #include <iconv.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <pwd.h>
+
 
 #include <curl/curl.h>
 
 namespace newsbeuter {
 
 std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::string delimiters) {
+	/*
+	 * This function tokenizes strings, obeying quotes and throwing away comments that start
+	 * with a '#'.
+	 *
+	 * e.g. line: foo bar "foo bar" "a test"
+	 * is parsed to 4 elements:
+	 * 	[0]: foo
+	 * 	[1]: bar
+	 * 	[2]: foo bar
+	 * 	[3]: a test
+	 *
+	 * e.g. line: yes great "x\ny" # comment
+	 * is parsed to 3 elements:
+	 * 	[0]: yes
+	 * 	[1]: great
+	 * 	[2]: x
+	 * 	y
+	 *
+	 * 	\", \r, \n, \t and \v are replaced with the literals that you know from C/C++ strings.
+	 *
+	 */
+	bool attach_backslash = true;
 	std::vector<std::string> tokens;
 	std::string::size_type last_pos = str.find_first_not_of(delimiters, 0);
 	std::string::size_type pos = last_pos;
@@ -23,15 +48,26 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 		if (str[last_pos] == '"') {
 			++last_pos;
 			pos = last_pos;
-			while (pos < str.length() && (str[pos] != '"' || str[pos-1] == '\\'))
+			int backslash_count = 0;
+			while (pos < str.length() && (str[pos] != '"' || (backslash_count%2))) {
+				if (str[pos] == '\\') {
+					++backslash_count;
+				} else {
+					backslash_count = 0;
+				}
 				++pos;
+			}
 			if (pos >= str.length()) {
 				pos = std::string::npos;
 				std::string token;
 				while (last_pos < str.length()) {
 					if (str[last_pos] == '\\') {
-						if (str[last_pos-1] == '\\')
-							token.append("\\");
+						if (str[last_pos-1] == '\\') {
+							if (attach_backslash) {
+								token.append("\\");
+							}
+							attach_backslash = !attach_backslash;
+						}
 					} else {
 						if (str[last_pos-1] == '\\') {
 							switch (str[last_pos]) {
@@ -53,8 +89,12 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 				std::string token;
 				while (last_pos < pos) {
 					if (str[last_pos] == '\\') {
-						if (str[last_pos-1] == '\\')
-							token.append("\\");
+						if (str[last_pos-1] == '\\') {
+							if (attach_backslash) {
+								token.append("\\");
+							}
+							attach_backslash = !attach_backslash;
+						}
 					} else {
 						if (str[last_pos-1] == '\\') {
 							switch (str[last_pos]) {
@@ -88,6 +128,9 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 
 	
 std::vector<std::string> utils::tokenize(const std::string& str, std::string delimiters) {
+	/*
+	 * This function tokenizes a string by the delimiters. Plain and simple.
+	 */
 	std::vector<std::string> tokens;
 	std::string::size_type last_pos = str.find_first_not_of(delimiters, 0);
 	std::string::size_type pos = str.find_first_of(delimiters, last_pos);
@@ -152,23 +195,34 @@ void utils::remove_fs_lock(const std::string& lock_file) {
 
 bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 	int fd;
-	if ((fd = ::open(lock_file.c_str(), O_WRONLY | O_CREAT | O_EXCL, 0600)) >= 0) {
-		pid = ::getpid();
-		::write(fd,&pid,sizeof(pid));
-		::close(fd);
-		GetLogger().log(LOG_DEBUG,"wrote lockfile %s with pid = %u",lock_file.c_str(), pid);
-		return true;
-	} else {
-		pid = 0;
-		if ((fd = ::open(lock_file.c_str(), O_RDONLY)) >=0) {
-			::read(fd,&pid,sizeof(pid));
-			::close(fd);
-			GetLogger().log(LOG_DEBUG,"found lockfile %s", lock_file.c_str());
-		} else {
-			GetLogger().log(LOG_DEBUG,"found lockfile %s, but couldn't open it for reading from it", lock_file.c_str());
-		}
+	// pid == 0 indicates that something went majorly wrong during locking
+	pid = 0;
+
+	// first, we open (and possibly create) the lock file
+	fd = ::open(lock_file.c_str(), O_RDWR | O_CREAT, 0600);
+	if (fd < 0)
 		return false;
+
+	// then we lock it (T_LOCK returns immediately if locking is not possible)
+	if (lockf(fd, F_TLOCK, 0) == 0) {
+		char buf[32];
+		snprintf(buf, sizeof(buf), "%u", getpid());
+		// locking successful -> truncate file and write own PID into it
+		ftruncate(fd, 0);
+		write(fd, buf, strlen(buf));
+		return true;
 	}
+
+	// locking was not successful -> read PID of locking process from it
+	fd = ::open(lock_file.c_str(), O_RDONLY);
+	if (fd >= 0) {
+		char buf[32];
+		int len = read(fd, buf, sizeof(buf)-1);
+		buf[len] = '\0';
+		sscanf(buf, "%u", &pid);
+		close(fd);
+	}
+	return false;
 }
 
 
@@ -266,7 +320,7 @@ static size_t my_write_data(void *buffer, size_t size, size_t nmemb, void *userp
 	return size * nmemb;
 }
 
-std::string utils::retrieve_url(const std::string& url, const char * user_agent) {
+std::string utils::retrieve_url(const std::string& url, const char * user_agent, const char * auth) {
 	std::string buf;
 
 	CURL * easyhandle = curl_easy_init();
@@ -276,6 +330,12 @@ std::string utils::retrieve_url(const std::string& url, const char * user_agent)
 	curl_easy_setopt(easyhandle, CURLOPT_URL, url.c_str());
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, &buf);
+
+	if (auth) {
+		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, auth);
+		curl_easy_setopt(easyhandle, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
+	}
+
 	curl_easy_perform(easyhandle);
 
 	GetLogger().log(LOG_DEBUG, "utils::retrieve_url(%s): %s", url.c_str(), buf.c_str());
@@ -283,6 +343,26 @@ std::string utils::retrieve_url(const std::string& url, const char * user_agent)
 	return buf;
 }
 
+void utils::run_command(const std::string& cmd, const std::string& input) {
+	int rc = fork();
+	switch (rc) {
+		case -1: break;
+		case 0: { // child:
+			int fd = ::open("/dev/null", O_RDWR);
+			close(0);
+			close(1);
+			close(2);
+			dup2(fd, 0);
+			dup2(fd, 1);
+			dup2(fd, 2);
+			GetLogger().log(LOG_DEBUG, "utils::run_command: %s '%s'", cmd.c_str(), input.c_str());
+			execlp(cmd.c_str(), cmd.c_str(), input.c_str(), NULL);
+			GetLogger().log(LOG_DEBUG, "utils::run_command: execlp of %s failed: %s", cmd.c_str(), strerror(errno));
+			exit(1);
+		}
+	}
+}
+
 std::string utils::run_filter(const std::string& cmd, const std::string& input) {
 	std::string buf;
 	int ipipe[2];
@@ -308,9 +388,9 @@ std::string utils::run_filter(const std::string& cmd, const std::string& input)
 				write(ipipe[1], input.c_str(), input.length());
 				close(ipipe[1]);
 				char cbuf[1024];
-				int rc;
-				while ((rc = read(opipe[0], cbuf, sizeof(cbuf))) > 0) {
-					buf.append(cbuf, rc);
+				int rc2;
+				while ((rc2 = read(opipe[0], cbuf, sizeof(cbuf))) > 0) {
+					buf.append(cbuf, rc2);
 				}
 				close(opipe[0]);
 			}
@@ -319,5 +399,88 @@ std::string utils::run_filter(const std::string& cmd, const std::string& input)
 	return buf;
 }
 
+std::string utils::run_program(char * argv[], const std::string& input) {
+	std::string buf;
+	int ipipe[2];
+	int opipe[2];
+	pipe(ipipe);
+	pipe(opipe);
+
+	int errfd = ::open("/dev/null", O_WRONLY);
+
+
+	int rc = fork();
+	switch (rc) {
+		case -1: break;
+		case 0: { // child:
+				close(ipipe[1]);
+				close(opipe[0]);
+				dup2(ipipe[0], 0);
+				dup2(opipe[1], 1);
+
+				close(2);
+				if (errfd != -1) dup2(errfd, 2);
+
+				execvp(argv[0], argv);
+				exit(1);
+			}
+			break;
+		default: {
+				close(ipipe[0]);
+				close(opipe[1]);
+				write(ipipe[1], input.c_str(), input.length());
+				close(ipipe[1]);
+				char cbuf[1024];
+				int rc2;
+				while ((rc2 = read(opipe[0], cbuf, sizeof(cbuf))) > 0) {
+					buf.append(cbuf, rc2);
+				}
+				close(opipe[0]);
+			}
+			break;
+	}
+	return buf;
+}
+
+std::string utils::resolve_tilde(const std::string& str) {
+	const char * homedir;
+	std::string filepath;
+
+	if (!(homedir = ::getenv("HOME"))) {
+		struct passwd * spw = ::getpwuid(::getuid());
+		if (spw) {
+				homedir = spw->pw_dir;
+		} else {
+				homedir = "";
+		}
+	}
+
+	if (strcmp(homedir,"")!=0) {
+		if (str == "~") {
+			filepath.append(homedir);
+		} else if (str.substr(0,2) == "~/") {
+			filepath.append(homedir);
+			filepath.append(1,'/');
+			filepath.append(str.substr(2,str.length()-2));
+		} else {
+			filepath.append(str);
+		}
+	} else {
+		filepath.append(str);
+	}
+
+	return filepath;
+}
+
+std::string utils::replace_all(std::string str, const std::string& from, const std::string& to) {
+	GetLogger().log(LOG_DEBUG,"utils::replace_all: before str = %s", str.c_str());
+	std::string::size_type s = str.find(from);
+	while (s != std::string::npos) {
+		str.replace(s,from.length(), to);
+		s = str.find(from, s + to.length());
+	}
+	GetLogger().log(LOG_DEBUG,"utils::replace_all: after str = %s", str.c_str());
+	return str;
+}
 
 }
